@@ -19,7 +19,7 @@
 mod tests {
 
     use crate::datasource::MemTable;
-    use crate::datasource::{provider_as_source, DefaultTableSource};
+    use crate::datasource::{DefaultTableSource, provider_as_source};
     use crate::physical_plan::collect;
     use crate::prelude::SessionContext;
     use arrow::array::{AsArray, Int32Array};
@@ -29,8 +29,8 @@ mod tests {
     use arrow_schema::SchemaRef;
     use datafusion_catalog::TableProvider;
     use datafusion_common::{DataFusionError, Result};
-    use datafusion_expr::dml::InsertOp;
     use datafusion_expr::LogicalPlanBuilder;
+    use datafusion_expr::dml::InsertOp;
     use futures::StreamExt;
     use std::collections::HashMap;
     use std::sync::Arc;
@@ -329,12 +329,11 @@ mod tests {
         );
         let col = batch.column(0).as_primitive::<UInt64Type>();
         assert_eq!(col.len(), 1, "expected 1 row, got {}", col.len());
-        let val = col
-            .iter()
+
+        col.iter()
             .next()
             .expect("had value")
-            .expect("expected non null");
-        val
+            .expect("expected non null")
     }
 
     // Test inserting a single batch of data into a single partition
