@@ -17,35 +17,33 @@
 
 use std::sync::Arc;
 
+use abi_stable::library::{LibraryError, RootModule};
+use abi_stable::prefix_type::PrefixTypeTrait;
+use abi_stable::sabi_types::VersionStrings;
 use abi_stable::{
-    declare_root_module_statics, export_root_module,
-    library::{LibraryError, RootModule},
-    package_version_strings,
-    prefix_type::PrefixTypeTrait,
-    sabi_types::VersionStrings,
-    StableAbi,
+    StableAbi, declare_root_module_statics, export_root_module, package_version_strings,
 };
-use catalog::create_catalog_provider;
-
-use crate::{catalog_provider::FFI_CatalogProvider, udtf::FFI_TableFunction};
-
-use crate::udaf::FFI_AggregateUDF;
-
-use crate::udwf::FFI_WindowUDF;
-
-use super::{table_provider::FFI_TableProvider, udf::FFI_ScalarUDF};
 use arrow::array::RecordBatch;
+use arrow_schema::{DataType, Field, Schema};
 use async_provider::create_async_table_provider;
-use datafusion::{
-    arrow::datatypes::{DataType, Field, Schema},
-    common::record_batch,
-};
+use catalog::create_catalog_provider;
+use datafusion_common::record_batch;
 use sync_provider::create_sync_table_provider;
 use udf_udaf_udwf::{
     create_ffi_abs_func, create_ffi_random_func, create_ffi_rank_func,
     create_ffi_stddev_func, create_ffi_sum_func, create_ffi_table_func,
 };
 
+use super::table_provider::FFI_TableProvider;
+use super::udf::FFI_ScalarUDF;
+use crate::catalog_provider::FFI_CatalogProvider;
+use crate::catalog_provider_list::FFI_CatalogProviderList;
+use crate::proto::logical_extension_codec::FFI_LogicalExtensionCodec;
+use crate::tests::catalog::create_catalog_provider_list;
+use crate::udaf::FFI_AggregateUDF;
+use crate::udtf::FFI_TableFunction;
+use crate::udwf::FFI_WindowUDF;
+
 mod async_provider;
 pub mod catalog;
 mod sync_provider;
@@ -60,17 +58,26 @@ pub mod utils;
 /// module.
 pub struct ForeignLibraryModule {
     /// Construct an opinionated catalog provider
-    pub create_catalog: extern "C" fn() -> FFI_CatalogProvider,
+    pub create_catalog:
+        extern "C" fn(codec: FFI_LogicalExtensionCodec) -> FFI_CatalogProvider,
+
+    /// Construct an opinionated catalog provider list
+    pub create_catalog_list:
+        extern "C" fn(codec: FFI_LogicalExtensionCodec) -> FFI_CatalogProviderList,
 
     /// Constructs the table provider
-    pub create_table: extern "C" fn(synchronous: bool) -> FFI_TableProvider,
+    pub create_table: extern "C" fn(
+        synchronous: bool,
+        codec: FFI_LogicalExtensionCodec,
+    ) -> FFI_TableProvider,
 
     /// Create a scalar UDF
     pub create_scalar_udf: extern "C" fn() -> FFI_ScalarUDF,
 
     pub create_nullary_udf: extern "C" fn() -> FFI_ScalarUDF,
 
-    pub create_table_function: extern "C" fn() -> FFI_TableFunction,
+    pub create_table_function:
+        extern "C" fn(FFI_LogicalExtensionCodec) -> FFI_TableFunction,
 
     /// Create an aggregate UDAF using sum
     pub create_sum_udaf: extern "C" fn() -> FFI_AggregateUDF,
@@ -111,10 +118,13 @@ pub fn create_record_batch(start_value: i32, num_values: usize) -> RecordBatch {
 
 /// Here we only wish to create a simple table provider as an example.
 /// We create an in-memory table and convert it to it's FFI counterpart.
-extern "C" fn construct_table_provider(synchronous: bool) -> FFI_TableProvider {
+extern "C" fn construct_table_provider(
+    synchronous: bool,
+    codec: FFI_LogicalExtensionCodec,
+) -> FFI_TableProvider {
     match synchronous {
-        true => create_sync_table_provider(),
-        false => create_async_table_provider(),
+        true => create_sync_table_provider(codec),
+        false => create_async_table_provider(codec),
     }
 }
 
@@ -123,6 +133,7 @@ extern "C" fn construct_table_provider(synchronous: bool) -> FFI_TableProvider {
 pub fn get_foreign_library_module() -> ForeignLibraryModuleRef {
     ForeignLibraryModule {
         create_catalog: create_catalog_provider,
+        create_catalog_list: create_catalog_provider_list,
         create_table: construct_table_provider,
         create_scalar_udf: create_ffi_abs_func,
         create_nullary_udf: create_ffi_random_func,
