@@ -20,10 +20,10 @@ use crate::utils::make_scalar_function;
 use arrow::array::{ArrayRef, GenericListArray, OffsetSizeTrait};
 use arrow::datatypes::DataType;
 use arrow::datatypes::DataType::{LargeList, List};
+use datafusion_common::Result;
 use datafusion_common::cast::{as_large_list_array, as_list_array};
 use datafusion_common::utils::take_function_args;
-use datafusion_common::Result;
-use datafusion_common::{exec_err, plan_err, ScalarValue};
+use datafusion_common::{ScalarValue, exec_err, plan_err};
 use datafusion_doc::Documentation;
 use datafusion_expr::{
     ColumnarValue, ScalarFunctionArgs, ScalarUDFImpl, Signature, Volatility,
@@ -113,14 +113,7 @@ impl ScalarUDFImpl for ArrayMax {
     }
 }
 
-/// array_max SQL function
-///
-/// There is one argument for array_max as the array.
-/// `array_max(array)`
-///
-/// For example:
-/// > array_max(\[1, 3, 2]) -> 3
-pub fn array_max_inner(args: &[ArrayRef]) -> Result<ArrayRef> {
+fn array_max_inner(args: &[ArrayRef]) -> Result<ArrayRef> {
     let [array] = take_function_args("array_max", args)?;
     match array.data_type() {
         List(_) => array_min_max_helper(as_list_array(array)?, max_batch),
@@ -202,7 +195,7 @@ impl ScalarUDFImpl for ArrayMin {
     }
 }
 
-pub fn array_min_inner(args: &[ArrayRef]) -> Result<ArrayRef> {
+fn array_min_inner(args: &[ArrayRef]) -> Result<ArrayRef> {
     let [array] = take_function_args("array_min", args)?;
     match array.data_type() {
         List(_) => array_min_max_helper(as_list_array(array)?, min_batch),
