@@ -23,28 +23,28 @@ use arrow::datatypes::TimeUnit;
 use arrow::datatypes::{DataType, Field, IntervalUnit::MonthDayNano};
 use arrow::{
     array::{
+        Array, ArrayRef, Int64Array, ListArray, ListBuilder, NullBufferBuilder,
         builder::{Date32Builder, TimestampNanosecondBuilder},
         temporal_conversions::as_datetime_with_timezone,
         timezone::Tz,
         types::{Date32Type, IntervalMonthDayNanoType, TimestampNanosecondType},
-        Array, ArrayRef, Int64Array, ListArray, ListBuilder, NullBufferBuilder,
     },
     compute::cast,
 };
 use datafusion_common::internal_err;
 use datafusion_common::{
+    Result, exec_datafusion_err, exec_err, not_impl_datafusion_err,
+    utils::take_function_args,
+};
+use datafusion_common::{
+    ScalarValue,
     cast::{
         as_date32_array, as_int64_array, as_interval_mdn_array,
         as_timestamp_nanosecond_array,
     },
     types::{
-        logical_date, logical_int64, logical_interval_mdn, logical_string, NativeType,
+        NativeType, logical_date, logical_int64, logical_interval_mdn, logical_string,
     },
-    ScalarValue,
-};
-use datafusion_common::{
-    exec_datafusion_err, exec_err, not_impl_datafusion_err, utils::take_function_args,
-    Result,
 };
 use datafusion_expr::{
     Coercion, ColumnarValue, Documentation, ScalarUDFImpl, Signature, TypeSignature,
@@ -573,7 +573,7 @@ fn gen_range_iter(
 }
 
 fn parse_tz(tz: &Option<&str>) -> Result<Tz> {
-    let tz = tz.as_ref().map_or_else(|| "+00", |s| s);
+    let tz = tz.unwrap_or_else(|| "+00");
 
     Tz::from_str(tz)
         .map_err(|op| exec_datafusion_err!("failed to parse timezone {tz}: {:?}", op))
