@@ -20,7 +20,7 @@ use arrow::datatypes::{DataType, Field, FieldRef, UnionFields};
 use datafusion_common::cast::as_union_array;
 use datafusion_common::utils::take_function_args;
 use datafusion_common::{
-    exec_datafusion_err, exec_err, internal_err, Result, ScalarValue,
+    Result, ScalarValue, exec_datafusion_err, exec_err, internal_err,
 };
 use datafusion_doc::Documentation;
 use datafusion_expr::{ColumnarValue, ReturnFieldArgs, ScalarFunctionArgs};
@@ -117,9 +117,16 @@ impl ScalarUDFImpl for UnionExtractFun {
         let [array, target_name] = take_function_args("union_extract", args.args)?;
 
         let target_name = match target_name {
-            ColumnarValue::Scalar(ScalarValue::Utf8(Some(target_name))) => Ok(target_name),
-            ColumnarValue::Scalar(ScalarValue::Utf8(None)) => exec_err!("union_extract second argument must be a non-null string literal, got a null instead"),
-            _ => exec_err!("union_extract second argument must be a non-null string literal, got {} instead", target_name.data_type()),
+            ColumnarValue::Scalar(ScalarValue::Utf8(Some(target_name))) => {
+                Ok(target_name)
+            }
+            ColumnarValue::Scalar(ScalarValue::Utf8(None)) => exec_err!(
+                "union_extract second argument must be a non-null string literal, got a null instead"
+            ),
+            _ => exec_err!(
+                "union_extract second argument must be a non-null string literal, got {} instead",
+                target_name.data_type()
+            ),
         }?;
 
         match array {
