@@ -15,13 +15,11 @@
 // specific language governing permissions and limitations
 // under the License.
 
-//! "crypto" DataFusion functions
 use crate::crypto::basic::md5;
 use arrow::datatypes::DataType;
 use datafusion_common::{
-    plan_err,
-    types::{logical_binary, logical_string, NativeType},
     Result,
+    types::{logical_binary, logical_string},
 };
 use datafusion_expr::{
     ColumnarValue, Documentation, ScalarFunctionArgs, ScalarUDFImpl, Signature,
@@ -37,11 +35,11 @@ use std::any::Any;
     syntax_example = "md5(expression)",
     sql_example = r#"```sql
 > select md5('foo');
-+-------------------------------------+
-| md5(Utf8("foo"))                    |
-+-------------------------------------+
-| <md5_checksum_result>               |
-+-------------------------------------+
++----------------------------------+
+| md5(Utf8("foo"))                 |
++----------------------------------+
+| acbd18db4cc2f85cedef654fccc4a4d8 |
++----------------------------------+
 ```"#,
     standard_argument(name = "expression", prefix = "String")
 )]
@@ -49,6 +47,7 @@ use std::any::Any;
 pub struct Md5Func {
     signature: Signature,
 }
+
 impl Default for Md5Func {
     fn default() -> Self {
         Self::new()
@@ -60,15 +59,11 @@ impl Md5Func {
         Self {
             signature: Signature::one_of(
                 vec![
-                    TypeSignature::Coercible(vec![Coercion::new_implicit(
-                        TypeSignatureClass::Native(logical_binary()),
-                        vec![TypeSignatureClass::Native(logical_string())],
-                        NativeType::String,
+                    TypeSignature::Coercible(vec![Coercion::new_exact(
+                        TypeSignatureClass::Native(logical_string()),
                     )]),
-                    TypeSignature::Coercible(vec![Coercion::new_implicit(
+                    TypeSignature::Coercible(vec![Coercion::new_exact(
                         TypeSignatureClass::Native(logical_binary()),
-                        vec![TypeSignatureClass::Native(logical_binary())],
-                        NativeType::Binary,
                     )]),
                 ],
                 Volatility::Immutable,
@@ -76,6 +71,7 @@ impl Md5Func {
         }
     }
 }
+
 impl ScalarUDFImpl for Md5Func {
     fn as_any(&self) -> &dyn Any {
         self
@@ -89,30 +85,10 @@ impl ScalarUDFImpl for Md5Func {
         &self.signature
     }
 
-    fn return_type(&self, arg_types: &[DataType]) -> Result<DataType> {
-        use DataType::*;
-        Ok(match &arg_types[0] {
-            LargeUtf8 | LargeBinary => Utf8View,
-            Utf8View | Utf8 | Binary | BinaryView => Utf8View,
-            Null => Null,
-            Dictionary(_, t) => match **t {
-                LargeUtf8 | LargeBinary => Utf8View,
-                Utf8 | Binary | BinaryView => Utf8View,
-                Null => Null,
-                _ => {
-                    return plan_err!(
-                        "the md5 can only accept strings but got {:?}",
-                        **t
-                    );
-                }
-            },
-            other => {
-                return plan_err!(
-                    "The md5 function can only accept strings. Got {other}"
-                );
-            }
-        })
+    fn return_type(&self, _arg_types: &[DataType]) -> Result<DataType> {
+        Ok(DataType::Utf8View)
     }
+
     fn invoke_with_args(&self, args: ScalarFunctionArgs) -> Result<ColumnarValue> {
         md5(&args.args)
     }
