@@ -119,7 +119,7 @@ mod tests {
     use arrow::datatypes::DataType::Int32;
 
     use datafusion_common::ScalarValue;
-    use datafusion_common::{exec_err, Result};
+    use datafusion_common::{Result, exec_err};
     use datafusion_expr::{ColumnarValue, ScalarUDFImpl};
 
     use crate::string::octet_length::OctetLengthFunc;
