@@ -19,10 +19,10 @@
 //! [`ExprSimplifier`] simplifies individual `Expr`s.
 
 pub mod expr_simplifier;
-mod guarantees;
 mod inlist_simplifier;
 mod regex;
 pub mod simplify_exprs;
+pub mod simplify_literal;
 mod simplify_predicates;
 mod unwrap_cast;
 mod utils;
@@ -35,4 +35,4 @@ pub use simplify_exprs::*;
 pub use simplify_predicates::simplify_predicates;
 
 // Export for test in datafusion/core/tests/optimizer_integration.rs
-pub use guarantees::GuaranteeRewriter;
+pub use datafusion_expr::expr_rewriter::GuaranteeRewriter;
