@@ -17,15 +17,15 @@
 
 //! A custom binary heap implementation for performant top K aggregation
 
+use arrow::array::{ArrayRef, ArrowPrimitiveType, PrimitiveArray, downcast_primitive};
 use arrow::array::{
     cast::AsArray,
     types::{IntervalDayTime, IntervalMonthDayNano},
 };
-use arrow::array::{downcast_primitive, ArrayRef, ArrowPrimitiveType, PrimitiveArray};
 use arrow::buffer::ScalarBuffer;
-use arrow::datatypes::{i256, DataType};
-use datafusion_common::exec_datafusion_err;
+use arrow::datatypes::{DataType, i256};
 use datafusion_common::Result;
+use datafusion_common::exec_datafusion_err;
 
 use half::f16;
 use std::cmp::Ordering;
@@ -72,7 +72,6 @@ pub trait ArrowHeap {
     fn set_batch(&mut self, vals: ArrayRef);
     fn is_worse(&self, idx: usize) -> bool;
     fn worst_map_idx(&self) -> usize;
-    fn renumber(&mut self, heap_to_map: &[(usize, usize)]);
     fn insert(&mut self, row_idx: usize, map_idx: usize, map: &mut Vec<(usize, usize)>);
     fn replace_if_better(
         &mut self,
@@ -131,10 +130,6 @@ where
         self.heap.worst_map_idx()
     }
 
-    fn renumber(&mut self, heap_to_map: &[(usize, usize)]) {
-        self.heap.renumber(heap_to_map);
-    }
-
     fn insert(&mut self, row_idx: usize, map_idx: usize, map: &mut Vec<(usize, usize)>) {
         let vals = self.batch.as_primitive::<VAL>();
         let new_val = vals.value(row_idx);
@@ -268,14 +263,6 @@ impl<VAL: ValueType> TopKHeap<VAL> {
         self.heapify_down(heap_idx, mapper);
     }
 
-    pub fn renumber(&mut self, heap_to_map: &[(usize, usize)]) {
-        for (heap_idx, map_idx) in heap_to_map.iter() {
-            if let Some(Some(hi)) = self.heap.get_mut(*heap_idx) {
-                hi.map_idx = *map_idx;
-            }
-        }
-    }
-
     fn heapify_up(&mut self, mut idx: usize, mapper: &mut Vec<(usize, usize)>) {
         let desc = self.desc;
         while idx != 0 {
@@ -311,13 +298,12 @@ impl<VAL: ValueType> TopKHeap<VAL> {
         let mut best_idx = node_idx;
         let mut best_val = &entry.val;
         for child_idx in left_child..=left_child + 1 {
-            if let Some(Some(child)) = self.heap.get(child_idx) {
-                if (!desc && child.val.comp(best_val) == Ordering::Greater)
-                    || (desc && child.val.comp(best_val) == Ordering::Less)
-                {
-                    best_val = &child.val;
-                    best_idx = child_idx;
-                }
+            if let Some(Some(child)) = self.heap.get(child_idx)
+                && ((!desc && child.val.comp(best_val) == Ordering::Greater)
+                    || (desc && child.val.comp(best_val) == Ordering::Less))
+            {
+                best_val = &child.val;
+                best_idx = child_idx;
             }
         }
         if best_val.comp(&entry.val) != Ordering::Equal {
@@ -326,20 +312,10 @@ impl<VAL: ValueType> TopKHeap<VAL> {
         }
     }
 
-    fn _tree_print(
-        &self,
-        idx: usize,
-        prefix: String,
-        is_tail: bool,
-        output: &mut String,
-    ) {
+    fn _tree_print(&self, idx: usize, prefix: &str, is_tail: bool, output: &mut String) {
         if let Some(Some(hi)) = self.heap.get(idx) {
             let connector = if idx != 0 {
-                if is_tail {
-                    "└── "
-                } else {
-                    "├── "
-                }
+                if is_tail { "└── " } else { "├── " }
             } else {
                 ""
             };
@@ -357,10 +333,10 @@ impl<VAL: ValueType> TopKHeap<VAL> {
             let right_exists = right_idx < self.len;
 
             if left_exists {
-                self._tree_print(left_idx, child_prefix.clone(), !right_exists, output);
+                self._tree_print(left_idx, &child_prefix, !right_exists, output);
             }
             if right_exists {
-                self._tree_print(right_idx, child_prefix, true, output);
+                self._tree_print(right_idx, &child_prefix, true, output);
             }
         }
     }
@@ -370,7 +346,7 @@ impl<VAL: ValueType> Display for TopKHeap<VAL> {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         let mut output = String::new();
         if !self.heap.is_empty() {
-            self._tree_print(0, String::new(), true, &mut output);
+            self._tree_print(0, "", true, &mut output);
         }
         write!(f, "{output}")
     }
@@ -494,9 +470,7 @@ mod tests {
         heap.append_or_replace(1, 1, &mut map);
 
         let actual = heap.to_string();
-        assert_snapshot!(actual, @r#"
-val=1 idx=0, bucket=1
-            "#);
+        assert_snapshot!(actual, @"val=1 idx=0, bucket=1");
 
         Ok(())
     }
@@ -513,10 +487,10 @@ val=1 idx=0, bucket=1
         assert_eq!(map, vec![(2, 0), (1, 1)]);
 
         let actual = heap.to_string();
-        assert_snapshot!(actual, @r#"
-val=2 idx=0, bucket=2
-└── val=1 idx=1, bucket=1
-            "#);
+        assert_snapshot!(actual, @r"
+        val=2 idx=0, bucket=2
+        └── val=1 idx=1, bucket=1
+        ");
 
         Ok(())
     }
@@ -530,20 +504,20 @@ val=2 idx=0, bucket=2
         heap.append_or_replace(2, 2, &mut map);
         heap.append_or_replace(3, 3, &mut map);
         let actual = heap.to_string();
-        assert_snapshot!(actual, @r#"
-val=3 idx=0, bucket=3
-├── val=1 idx=1, bucket=1
-└── val=2 idx=2, bucket=2
-            "#);
+        assert_snapshot!(actual, @r"
+        val=3 idx=0, bucket=3
+        ├── val=1 idx=1, bucket=1
+        └── val=2 idx=2, bucket=2
+        ");
 
         let mut map = vec![];
         heap.append_or_replace(0, 0, &mut map);
         let actual = heap.to_string();
-        assert_snapshot!(actual, @r#"
-val=2 idx=0, bucket=2
-├── val=1 idx=1, bucket=1
-└── val=0 idx=2, bucket=0
-            "#);
+        assert_snapshot!(actual, @r"
+        val=2 idx=0, bucket=2
+        ├── val=1 idx=1, bucket=1
+        └── val=0 idx=2, bucket=0
+        ");
         assert_eq!(map, vec![(2, 0), (0, 2)]);
 
         Ok(())
@@ -559,22 +533,22 @@ val=2 idx=0, bucket=2
         heap.append_or_replace(3, 3, &mut map);
         heap.append_or_replace(4, 4, &mut map);
         let actual = heap.to_string();
-        assert_snapshot!(actual, @r#"
-val=4 idx=0, bucket=4
-├── val=3 idx=1, bucket=3
-│   └── val=1 idx=3, bucket=1
-└── val=2 idx=2, bucket=2
-            "#);
+        assert_snapshot!(actual, @r"
+        val=4 idx=0, bucket=4
+        ├── val=3 idx=1, bucket=3
+        │   └── val=1 idx=3, bucket=1
+        └── val=2 idx=2, bucket=2
+        ");
 
         let mut map = vec![];
         heap.replace_if_better(1, 0, &mut map);
         let actual = heap.to_string();
-        assert_snapshot!(actual, @r#"
-val=4 idx=0, bucket=4
-├── val=1 idx=1, bucket=1
-│   └── val=0 idx=3, bucket=3
-└── val=2 idx=2, bucket=2
-            "#);
+        assert_snapshot!(actual, @r"
+        val=4 idx=0, bucket=4
+        ├── val=1 idx=1, bucket=1
+        │   └── val=0 idx=3, bucket=3
+        └── val=2 idx=2, bucket=2
+        ");
         assert_eq!(map, vec![(1, 1), (3, 3)]);
 
         Ok(())
@@ -589,10 +563,10 @@ val=4 idx=0, bucket=4
         heap.append_or_replace(2, 2, &mut map);
 
         let actual = heap.to_string();
-        assert_snapshot!(actual, @r#"
-val=2 idx=0, bucket=2
-└── val=1 idx=1, bucket=1
-            "#);
+        assert_snapshot!(actual, @r"
+        val=2 idx=0, bucket=2
+        └── val=1 idx=1, bucket=1
+        ");
 
         assert_eq!(heap.worst_val(), Some(&2));
         assert_eq!(heap.worst_map_idx(), 2);
@@ -609,10 +583,10 @@ val=2 idx=0, bucket=2
         heap.append_or_replace(2, 2, &mut map);
 
         let actual = heap.to_string();
-        assert_snapshot!(actual, @r#"
-val=2 idx=0, bucket=2
-└── val=1 idx=1, bucket=1
-            "#);
+        assert_snapshot!(actual, @r"
+        val=2 idx=0, bucket=2
+        └── val=1 idx=1, bucket=1
+        ");
 
         let (vals, map_idxs) = heap.drain();
         assert_eq!(vals, vec![1, 2]);
@@ -621,29 +595,4 @@ val=2 idx=0, bucket=2
 
         Ok(())
     }
-
-    #[test]
-    fn should_renumber() -> Result<()> {
-        let mut map = vec![];
-        let mut heap = TopKHeap::new(10, false);
-
-        heap.append_or_replace(1, 1, &mut map);
-        heap.append_or_replace(2, 2, &mut map);
-
-        let actual = heap.to_string();
-        assert_snapshot!(actual, @r#"
-val=2 idx=0, bucket=2
-└── val=1 idx=1, bucket=1
-            "#);
-
-        let numbers = vec![(0, 1), (1, 2)];
-        heap.renumber(numbers.as_slice());
-        let actual = heap.to_string();
-        assert_snapshot!(actual, @r#"
-val=2 idx=0, bucket=1
-└── val=1 idx=1, bucket=2
-            "#);
-
-        Ok(())
-    }
 }
