@@ -23,10 +23,10 @@ use arrow::{
 };
 use arrow_schema::{SchemaRef, SortOptions};
 use datafusion_common::not_impl_err;
-use datafusion_common::{internal_err, JoinSide, Result};
+use datafusion_common::{JoinSide, Result, internal_err};
 use datafusion_execution::{
-    memory_pool::{MemoryConsumer, MemoryReservation},
     SendableRecordBatchStream,
+    memory_pool::{MemoryConsumer, MemoryReservation},
 };
 use datafusion_expr::{JoinType, Operator};
 use datafusion_physical_expr::equivalence::join_equivalence_properties;
@@ -38,10 +38,10 @@ use datafusion_physical_expr_common::physical_expr::fmt_sql;
 use futures::TryStreamExt;
 use parking_lot::Mutex;
 use std::fmt::Formatter;
-use std::sync::atomic::AtomicUsize;
 use std::sync::Arc;
+use std::sync::atomic::AtomicUsize;
 
-use crate::execution_plan::{boundedness_from_children, EmissionType};
+use crate::execution_plan::{EmissionType, boundedness_from_children};
 
 use crate::joins::piecewise_merge_join::classic_join::{
     ClassicPWMJStream, PiecewiseMergeJoinStreamState,
@@ -50,16 +50,17 @@ use crate::joins::piecewise_merge_join::utils::{
     build_visited_indices_map, is_existence_join, is_right_existence_join,
 };
 use crate::joins::utils::asymmetric_join_output_partitioning;
+use crate::metrics::MetricsSet;
+use crate::{DisplayAs, DisplayFormatType, ExecutionPlanProperties};
 use crate::{
+    ExecutionPlan, PlanProperties,
     joins::{
-        utils::{build_join_schema, BuildProbeJoinMetrics, OnceAsync, OnceFut},
         SharedBitmapBuilder,
+        utils::{BuildProbeJoinMetrics, OnceAsync, OnceFut, build_join_schema},
     },
     metrics::ExecutionPlanMetricsSet,
     spill::get_record_batch_memory_size,
-    ExecutionPlan, PlanProperties,
 };
-use crate::{DisplayAs, DisplayFormatType, ExecutionPlanProperties};
 
 /// `PiecewiseMergeJoinExec` is a join execution plan that only evaluates single range filter and show much
 /// better performance for these workloads than `NestedLoopJoin`
@@ -272,7 +273,7 @@ pub struct PiecewiseMergeJoinExec {
     left_child_plan_required_order: LexOrdering,
     /// The right sort order, descending for `<`, `<=` operations + ascending for `>`, `>=` operations
     /// Unsorted for mark joins
-    #[allow(unused)]
+    #[expect(dead_code)]
     right_batch_required_orders: LexOrdering,
 
     /// This determines the sort order of all join columns used in sorting the stream and buffered execution plans.
@@ -321,7 +322,7 @@ impl PiecewiseMergeJoinExec {
             _ => {
                 return internal_err!(
                     "Cannot contain non-range operator in PiecewiseMergeJoinExec"
-                )
+                );
             }
         };
 
@@ -572,6 +573,10 @@ impl ExecutionPlan for PiecewiseMergeJoinExec {
             )))
         }
     }
+
+    fn metrics(&self) -> Option<MetricsSet> {
+        Some(self.metrics.clone_inner())
+    }
 }
 
 impl DisplayAs for PiecewiseMergeJoinExec {
