@@ -27,11 +27,11 @@ use crate::sorts::{
 use crate::{SendableRecordBatchStream, SpillManager};
 use arrow::array::*;
 use arrow::datatypes::{DataType, SchemaRef};
-use datafusion_common::{internal_err, Result};
+use datafusion_common::human_readable_size;
+use datafusion_common::{Result, assert_or_internal_err, internal_err};
 use datafusion_execution::disk_manager::RefCountedTempFile;
 use datafusion_execution::memory_pool::{
-    human_readable_size, MemoryConsumer, MemoryPool, MemoryReservation,
-    UnboundedMemoryPool,
+    MemoryConsumer, MemoryPool, MemoryReservation, UnboundedMemoryPool,
 };
 use datafusion_physical_expr_common::sort_expr::LexOrdering;
 use std::sync::Arc;
@@ -213,11 +213,10 @@ impl<'a> StreamingMergeBuilder<'a> {
         }
 
         // Early return if streams are empty:
-        if streams.is_empty() {
-            return internal_err!(
-                "Streams/sorted spill files cannot be empty for streaming merge"
-            );
-        }
+        assert_or_internal_err!(
+            !streams.is_empty(),
+            "Streams/sorted spill files cannot be empty for streaming merge"
+        );
 
         // Unwrapping mandatory fields
         let schema = schema.expect("Schema cannot be empty for streaming merge");
