@@ -20,15 +20,15 @@ use std::sync::Arc;
 
 use arrow::array::ArrayRef;
 use arrow::compute;
-use arrow::datatypes::{DataType, Date32Type};
-use arrow::error::ArrowError;
+use arrow::datatypes::{DataType, Date32Type, Field, FieldRef};
 use datafusion_common::cast::{
-    as_date32_array, as_int16_array, as_int32_array, as_int8_array,
+    as_date32_array, as_int8_array, as_int16_array, as_int32_array,
 };
-use datafusion_common::{internal_err, Result};
+use datafusion_common::utils::take_function_args;
+use datafusion_common::{Result, internal_err};
 use datafusion_expr::{
-    ColumnarValue, ScalarFunctionArgs, ScalarUDFImpl, Signature, TypeSignature,
-    Volatility,
+    ColumnarValue, ReturnFieldArgs, ScalarFunctionArgs, ScalarUDFImpl, Signature,
+    TypeSignature, Volatility,
 };
 use datafusion_functions::utils::make_scalar_function;
 
@@ -78,7 +78,21 @@ impl ScalarUDFImpl for SparkDateAdd {
     }
 
     fn return_type(&self, _arg_types: &[DataType]) -> Result<DataType> {
-        Ok(DataType::Date32)
+        internal_err!("Use return_field_from_args in this case instead.")
+    }
+
+    fn return_field_from_args(&self, args: ReturnFieldArgs) -> Result<FieldRef> {
+        let nullable = args.arg_fields.iter().any(|f| f.is_nullable())
+            || args
+                .scalar_arguments
+                .iter()
+                .any(|arg| matches!(arg, Some(sv) if sv.is_null()));
+
+        Ok(Arc::new(Field::new(
+            self.name(),
+            DataType::Date32,
+            nullable,
+        )))
     }
 
     fn invoke_with_args(&self, args: ScalarFunctionArgs) -> Result<ColumnarValue> {
@@ -87,48 +101,31 @@ impl ScalarUDFImpl for SparkDateAdd {
 }
 
 fn spark_date_add(args: &[ArrayRef]) -> Result<ArrayRef> {
-    let [date_arg, days_arg] = args else {
-        return internal_err!(
-            "Spark `date_add` function requires 2 arguments, got {}",
-            args.len()
-        );
-    };
+    let [date_arg, days_arg] = take_function_args("date_add", args)?;
     let date_array = as_date32_array(date_arg)?;
     let result = match days_arg.data_type() {
         DataType::Int8 => {
             let days_array = as_int8_array(days_arg)?;
-            compute::try_binary::<_, _, _, Date32Type>(
+            compute::binary::<_, _, _, Date32Type>(
                 date_array,
                 days_array,
-                |date, days| {
-                    date.checked_add(days as i32).ok_or_else(|| {
-                        ArrowError::ArithmeticOverflow("date_add".to_string())
-                    })
-                },
+                |date, days| date.wrapping_add(days as i32),
             )?
         }
         DataType::Int16 => {
             let days_array = as_int16_array(days_arg)?;
-            compute::try_binary::<_, _, _, Date32Type>(
+            compute::binary::<_, _, _, Date32Type>(
                 date_array,
                 days_array,
-                |date, days| {
-                    date.checked_add(days as i32).ok_or_else(|| {
-                        ArrowError::ArithmeticOverflow("date_add".to_string())
-                    })
-                },
+                |date, days| date.wrapping_add(days as i32),
             )?
         }
         DataType::Int32 => {
             let days_array = as_int32_array(days_arg)?;
-            compute::try_binary::<_, _, _, Date32Type>(
+            compute::binary::<_, _, _, Date32Type>(
                 date_array,
                 days_array,
-                |date, days| {
-                    date.checked_add(days).ok_or_else(|| {
-                        ArrowError::ArithmeticOverflow("date_add".to_string())
-                    })
-                },
+                |date, days| date.wrapping_add(days),
             )?
         }
         _ => {
@@ -140,3 +137,69 @@ fn spark_date_add(args: &[ArrayRef]) -> Result<ArrayRef> {
     };
     Ok(Arc::new(result))
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use arrow::datatypes::Field;
+    use datafusion_common::ScalarValue;
+
+    #[test]
+    fn test_date_add_non_nullable_inputs() {
+        let func = SparkDateAdd::new();
+        let args = &[
+            Arc::new(Field::new("date", DataType::Date32, false)),
+            Arc::new(Field::new("num", DataType::Int8, false)),
+        ];
+
+        let ret_field = func
+            .return_field_from_args(ReturnFieldArgs {
+                arg_fields: args,
+                scalar_arguments: &[None, None],
+            })
+            .unwrap();
+
+        assert_eq!(ret_field.data_type(), &DataType::Date32);
+        assert!(!ret_field.is_nullable());
+    }
+
+    #[test]
+    fn test_date_add_nullable_inputs() {
+        let func = SparkDateAdd::new();
+        let args = &[
+            Arc::new(Field::new("date", DataType::Date32, false)),
+            Arc::new(Field::new("num", DataType::Int16, true)),
+        ];
+
+        let ret_field = func
+            .return_field_from_args(ReturnFieldArgs {
+                arg_fields: args,
+                scalar_arguments: &[None, None],
+            })
+            .unwrap();
+
+        assert_eq!(ret_field.data_type(), &DataType::Date32);
+        assert!(ret_field.is_nullable());
+    }
+
+    #[test]
+    fn test_date_add_null_scalar() {
+        let func = SparkDateAdd::new();
+        let args = &[
+            Arc::new(Field::new("date", DataType::Date32, false)),
+            Arc::new(Field::new("num", DataType::Int32, false)),
+        ];
+
+        let null_scalar = ScalarValue::Int32(None);
+
+        let ret_field = func
+            .return_field_from_args(ReturnFieldArgs {
+                arg_fields: args,
+                scalar_arguments: &[None, Some(&null_scalar)],
+            })
+            .unwrap();
+
+        assert_eq!(ret_field.data_type(), &DataType::Date32);
+        assert!(ret_field.is_nullable());
+    }
+}
