@@ -38,10 +38,10 @@ mod tests {
 
         assert_snapshot!(
         plan,
-        @r#"
-            Projection: DATA.A AS a, DATA.B AS b, DATA.A + Int64(1) AS add1
-              TableScan: DATA
-            "#
+        @r"
+        Projection: DATA.A AS a, DATA.B AS b, DATA.A + Int64(1) AS add1
+          TableScan: DATA
+        "
                 );
         Ok(())
     }
@@ -57,11 +57,11 @@ mod tests {
         assert_snapshot!(
         plan,
         // Note that duplicate references in the remap are aliased
-        @r#"
-            Projection: DATA.B, DATA.A AS A1, DATA.A AS DATA.A__temp__0 AS A2
-              Filter: DATA.B = Int64(2)
-                TableScan: DATA
-            "#
+        @r"
+        Projection: DATA.B, DATA.A AS A1, DATA.A AS DATA.A__temp__0 AS A2
+          Filter: DATA.B = Int64(2)
+            TableScan: DATA
+        "
                 );
         Ok(())
     }
@@ -88,21 +88,21 @@ mod tests {
         let plan = df.into_unoptimized_plan();
         assert_snapshot!(
             plan,
-            @r#"
-            Projection: random() AS c1, data.a + Int64(1) AS c2
-              TableScan: data
-            "#        );
+            @r"
+        Projection: random() AS c1, data.a + Int64(1) AS c2
+          TableScan: data
+        "        );
 
         let proto = to_substrait_plan(&plan, &ctx.state())?;
         let plan2 = from_substrait_plan(&ctx.state(), &proto).await?;
         // note how the Projections are not flattened
         assert_snapshot!(
         plan2,
-        @r#"
-            Projection: random() AS c1, data.a + Int64(1) AS c2
-              Projection: data.a, data.b, data.c, data.d, data.e, data.f, random(), data.a + Int64(1)
-                TableScan: data
-            "#
+        @r"
+        Projection: random() AS c1, data.a + Int64(1) AS c2
+          Projection: data.a, data.b, data.c, data.d, data.e, data.f, random(), data.a + Int64(1)
+            TableScan: data
+        "
                 );
         Ok(())
     }
@@ -115,10 +115,10 @@ mod tests {
         let plan = df.into_unoptimized_plan();
         assert_snapshot!(
         plan,
-        @r#"
-            Projection: data.a + Int64(1), data.b + Int64(2)
-              TableScan: data
-            "#
+        @r"
+        Projection: data.a + Int64(1), data.b + Int64(2)
+          TableScan: data
+        "
                 );
 
         let proto = to_substrait_plan(&plan, &ctx.state())?;
