@@ -15,8 +15,15 @@
 // specific language governing permissions and limitations
 // under the License.
 
+mod binary;
+mod boolean;
+mod decimal;
 mod primitive;
+mod random_data;
 mod string;
 
+pub use binary::BinaryArrayGenerator;
+pub use boolean::BooleanArrayGenerator;
+pub use decimal::DecimalArrayGenerator;
 pub use primitive::PrimitiveArrayGenerator;
 pub use string::StringArrayGenerator;
